# Generated by Django 2.2.19 on 2021-09-14 18:53

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mellon', '0002_sessionindex'),
    ]

    operations = [
        migrations.CreateModel(
            name='Issuer',
            fields=[
                (
                    'id',
                    models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
                ),
                (
                    'entity_id',
                    models.TextField(unique=True, verbose_name='IdP Entity ID'),
                ),
                ('slug', models.TextField(null=True, unique=True, verbose_name='IdP slug')),
            ],
            options={
                'verbose_name': 'SAML IdP',
                'verbose_name_plural': 'SAML IdPs',
            },
        ),
        migrations.AlterField(
            model_name='usersamlidentifier',
            name='issuer',
            field=models.TextField(null=True, verbose_name='Issuer'),
        ),
        migrations.AddField(
            model_name='usersamlidentifier',
            name='issuer_fk',
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to='mellon.Issuer',
                verbose_name='Issuer',
            ),
        ),
    ]



# django-mellon - SAML2 authentication for Django
# Copyright (C) 2014-2019 Entr'ouvert
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from importlib import import_module

from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _


class UserSAMLIdentifier(models.Model):
    user = models.ForeignKey(
        verbose_name=_('user'),
        to=settings.AUTH_USER_MODEL,
        related_name='saml_identifiers',
        on_delete=models.CASCADE,
    )
    name_id = models.TextField(verbose_name=_('SAML identifier'))
    created = models.DateTimeField(verbose_name=_('created'), auto_now_add=True)
    issuer = models.ForeignKey('mellon.Issuer', verbose_name=_('Issuer'), null=True, on_delete=models.CASCADE)

    class Meta:
        verbose_name = _('user SAML identifier')
        verbose_name_plural = _('users SAML identifiers')
        unique_together = (('issuer', 'name_id'),)


class SessionIndex(models.Model):
    session_index = models.TextField(_('SAML SessionIndex'))
    session_key = models.CharField(_('Django session key'), max_length=40)
    saml_identifier = models.ForeignKey(
        verbose_name=_('SAML identifier'), to=UserSAMLIdentifier, on_delete=models.CASCADE
    )

    @staticmethod
    def cleanup(cls):
        session_engine = import_module(settings.SESSION_ENGINE)
        store = session_engine.SessionStore()

        ids = []
        for si in cls.objects.all():
            if not store.exists(si.session_key):
                ids.append(si.id)
        cls.objects.filter(id__in=ids).delete()

    class Meta:
        verbose_name = _('SAML SessionIndex')
        verbose_name_plural = _('SAML SessionIndexes')
        unique_together = (('saml_identifier', 'session_index', 'session_key'),)


class Issuer(models.Model):
    entity_id = models.TextField(verbose_name=_('IdP Entity ID'), unique=True)
    slug = models.TextField(verbose_name=_('IdP slug'), unique=True, null=True)

    class Meta:
        verbose_name = _('SAML IdP')
        verbose_name_plural = _('SAML IdPs')



# django-mellon - SAML2 authentication for Django
# Copyright (C) 2014-2019 Entr'ouvert
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from lxml import etree as ET


def assert_xml_constraints(content, constraint, namespaces={}):
    d = ET.fromstring(content)

    def check(constraint, prefix=''):
        path, count = constraint[:2]
        path = prefix + path
        if not count is None:
            l = d.xpath(path, namespaces=namespaces)
            assert len(l) == count, 'len(xpath(%s)) is not %s but %s' % (path, count, len(l))
        if len(constraint) > 2:
            for constraint in constraint[2:]:
                check(constraint, prefix=path)

    check(constraint)



# django-mellon - SAML2 authentication for Django
# Copyright (C) 2014-2019 Entr'ouvert
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.contrib.sessions.backends.db import SessionStore

from mellon import utils


class SessionStore(SessionStore):
    def get_session_not_on_or_after(self):
        session_not_on_or_after = self.get('mellon_session', {}).get('session_not_on_or_after')
        if session_not_on_or_after:
            return utils.iso8601_to_datetime(session_not_on_or_after)
        return None

    def get_expiry_age(self, **kwargs):
        session_not_on_or_after = self.get_session_not_on_or_after()
        if session_not_on_or_after and 'expiry' not in kwargs:
            kwargs['expiry'] = session_not_on_or_after
        return super().get_expiry_age(**kwargs)

    def get_expiry_date(self, **kwargs):
        session_not_on_or_after = self.get_session_not_on_or_after()
        if session_not_on_or_after and 'expiry' not in kwargs:
            kwargs['expiry'] = session_not_on_or_after
        return super().get_expiry_date(**kwargs)



# django-mellon - SAML2 authentication for Django
# Copyright (C) 2014-2021 Entr'ouvert
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pytest
from django.contrib.auth.models import User

from mellon.models import Issuer, UserSAMLIdentifier


@pytest.fixture
def user_and_issuers(db):
    user1 = User.objects.create(username='user1')
    user2 = User.objects.create(username='user2')
    issuer1 = Issuer.objects.create(entity_id='https://idp1')
    issuer2 = Issuer.objects.create(entity_id='https://idp2')
    UserSAMLIdentifier.objects.create(user=user1, issuer=issuer1, name_id='xxx')
    UserSAMLIdentifier.objects.create(user=user2, issuer=issuer2, name_id='yyy')


def test_migration_0004_migrate_issuer_back_and_forward(transactional_db, user_and_issuers, migration):
    migration.before([('mellon', '0002_sessionindex')])
    new_apps = migration.apply([('mellon', '0004_migrate_issuer')])

    UserSAMLIdentifier = new_apps.get_model('mellon', 'UserSAMLIdentifier')
    Issuer = new_apps.get_model('mellon', 'Issuer')
    User = new_apps.get_model('auth', 'User')

    user1 = User.objects.get(username='user1')
    user2 = User.objects.get(username='user2')

    assert UserSAMLIdentifier.objects.count() == 2
    assert Issuer.objects.count() == 2
    assert UserSAMLIdentifier.objects.get(user=user1, issuer_fk__entity_id='https://idp1', name_id='xxx')
    assert UserSAMLIdentifier.objects.get(user=user2, issuer_fk__entity_id='https://idp2', name_id='yyy')


def test_migration_0004_migrate_issuer(transactional_db, migration):
    old_apps = migration.before([('mellon', '0003_add_issuer_model')])

    UserSAMLIdentifier = old_apps.get_model('mellon', 'UserSAMLIdentifier')
    User = old_apps.get_model('auth', 'User')

    user1 = User.objects.create(username='user1')
    user2 = User.objects.create(username='user2')

    UserSAMLIdentifier.objects.create(user=user1, issuer='https://idp1', name_id='xxx')
    UserSAMLIdentifier.objects.create(user=user2, issuer='https://idp2', name_id='yyy')

    new_apps = migration.apply([('mellon', '0004_migrate_issuer')])

    UserSAMLIdentifier = new_apps.get_model('mellon', 'UserSAMLIdentifier')
    Issuer = new_apps.get_model('mellon', 'Issuer')
    User = new_apps.get_model('auth', 'User')

    user1 = User.objects.get(username='user1')
    user2 = User.objects.get(username='user2')

    assert UserSAMLIdentifier.objects.count() == 2
    assert Issuer.objects.count() == 2
    assert UserSAMLIdentifier.objects.get(user=user1, issuer_fk__entity_id='https://idp1', name_id='xxx')
    assert UserSAMLIdentifier.objects.get(user=user2, issuer_fk__entity_id='https://idp2', name_id='yyy')



# django-mellon - SAML2 authentication for Django
# Copyright (C) 2014-2019 Entr'ouvert
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.conf.urls import include, url
from django.http import HttpResponse


def homepage(request):
    return HttpResponse('ok')


def context_hook(context):
    context['hook'] = 'HOOK'


urlpatterns = [
    url(
        '^',
        include('mellon.urls'),
        kwargs={
            'template_base': 'theme.html',
            'context_hook': context_hook,
        },
    ),
    url('^$', homepage, name='homepage'),
]



import django
from django.conf.urls import url

from . import views

urlpatterns = [
    url('login/$', views.login, name='mellon_login'),
    url('login/debug/$', views.debug_login, name='mellon_debug_login'),
    url('logout/$', views.logout, name='mellon_logout'),
    url('metadata/$', views.metadata, name='mellon_metadata'),
]



from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='UserSAMLIdentifier',
            fields=[
                (
                    'id',
                    models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True),
                ),
                ('issuer', models.TextField(verbose_name='Issuer')),
                ('name_id', models.TextField(verbose_name='SAML identifier')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='created')),
                (
                    'user',
                    models.ForeignKey(
                        related_name='saml_identifiers',
                        verbose_name='user',
                        to=settings.AUTH_USER_MODEL,
                        on_delete=models.CASCADE,
                    ),
                ),
            ],
            options={
                'verbose_name': 'user SAML identifier',
                'verbose_name_plural': 'users SAML identifiers',
            },
            bases=(models.Model,),
        ),
        migrations.AlterUniqueTogether(
            name='usersamlidentifier',
            unique_together={('issuer', 'name_id')},
        ),
    ]
